>From 045f070b3ca774738a1da2b10cf59df87628824e Mon Sep 17 00:00:00 2001
From: Roiy Shpaner <roiy@cs.umanitoba.ca>
Date: Tue, 15 Apr 2014 19:09:27 -0500
Subject: [PATCH 06/10] MED Support - Slider changes

---
 modules/gui/qt4/adapters/seeksegments.cpp | 119 +++++++++++++++++
 modules/gui/qt4/adapters/seeksegments.hpp |  98 ++++++++++++++
 modules/gui/qt4/input_manager.hpp         |   9 +-
 modules/gui/qt4/styles/seekstyle.cpp      | 127 +++++++++++++++---
 modules/gui/qt4/styles/seekstyle.hpp      |  12 +-
 modules/gui/qt4/util/input_slider.cpp     | 213 +++++++++++++++++++++++++++++-
 modules/gui/qt4/util/input_slider.hpp     |  46 +++++--
 7 files changed, 583 insertions(+), 41 deletions(-)
 create mode 100644 modules/gui/qt4/adapters/seeksegments.cpp
 create mode 100644 modules/gui/qt4/adapters/seeksegments.hpp

diff --git a/modules/gui/qt4/adapters/seeksegments.cpp b/modules/gui/qt4/adapters/seeksegments.cpp
new file mode 100644
index 0000000..aea77b7
--- /dev/null
+++ b/modules/gui/qt4/adapters/seeksegments.cpp
@@ -0,0 +1,119 @@
+/*****************************************************************************
+ * seeksegments.cpp : Segments for MetaLayers
+ *****************************************************************************
+ * Copyright © 2014 the VideoLAN team
+ *
+ * Author: Roiy Shpaner <roiy@cs.umanitoba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * ( at your option ) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+
+#include "recents.hpp"
+#include "dialogs_provider.hpp"
+#include "menus.hpp"
+
+#include "seeksegments.hpp"
+
+#include "qt4.hpp"
+#include "input_manager.hpp"
+
+SeekSegments::SeekSegments( QObject *parent, intf_thread_t *p_intf_ ) :
+    QObject( parent ), p_intf( p_intf_ )
+{
+    psz_layerName = NULL;
+    b_med_active = false;
+}
+
+SeekSegments::~SeekSegments()
+{
+    free( psz_layerName );
+}
+
+
+void SeekSegments::update( bool isMain )
+{
+    input_thread_t *p_input = playlist_CurrentInput( THEPL );
+    seeksegment_t **seeksegments;
+    int res, i_seeksegments;
+
+    // lock here too, as update event is triggered by an external thread
+    if ( !access() )
+    {
+        vlc_object_release( p_input );
+        return;
+    }
+
+    segmentsList.clear();
+
+    if( !p_input )
+    {
+        release();
+        return;
+    } 
+
+    bool bSelectedUnused;
+    input_Control( p_input, INPUT_GET_MED_STATE, &b_med_active, &bSelectedUnused );
+
+    if ( !b_med_active )
+    {
+        vlc_object_release( p_input );
+        release();
+        return;
+    }
+
+    int64_t iUpdateTime = -1;  /* Set MED to update after new state of events */
+    input_Control( p_input, INPUT_SET_MED_UPDATE_TIME, &iUpdateTime, &iUpdateTime );
+
+    if ( isMain )
+        res = input_Control( p_input, INPUT_GET_SELECTED_SEGMENTS, &seeksegments, &i_seeksegments );
+    else
+        res = input_Control( p_input, INPUT_GET_SEGMENTS, &seeksegments, &i_seeksegments, psz_layerName );
+
+    vlc_object_release( p_input );
+
+    if ( res != VLC_SUCCESS )
+    {
+        release();
+        return;
+    }
+    
+    for ( int i=0; i < i_seeksegments ; i++ )
+    {
+        segmentsList << SeekSegment( seeksegments[i] );
+    }
+        
+    vlc_seeksegment_Delete_arr( seeksegments, i_seeksegments );
+    
+    release();
+}
+
+void SeekSegments::updateMain()
+{
+    update( true );
+}
+
+QList<SeekSegment> const SeekSegments::getSegments()
+{
+    QList<SeekSegment> copy;
+    if ( access() )
+    {
+        copy = segmentsList;
+        release();
+    }
+
+    return copy;
+}
+
diff --git a/modules/gui/qt4/adapters/seeksegments.hpp b/modules/gui/qt4/adapters/seeksegments.hpp
new file mode 100644
index 0000000..bd46e37
--- /dev/null
+++ b/modules/gui/qt4/adapters/seeksegments.hpp
@@ -0,0 +1,98 @@
+/*****************************************************************************
+ * seekpoints.hpp : Segments for MetaLayers
+ *****************************************************************************
+ * Copyright © 2014 the VideoLAN team
+ *
+ * Author: Roiy Shpaner <roiy@cs.umanitoba.ca>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * ( at your option ) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#ifndef SEEKSEGMENTS_HPP
+#define SEEKSEGMENTS_HPP
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_interface.h>
+#include <vlc_input.h>
+
+#include <QColor>
+#include <QObject>
+#include <QList>
+#include <QMutex>
+
+static const QColor segmentColors[] = 
+{       
+        Qt::darkGreen,
+        Qt::blue,
+        Qt::red,
+        Qt::darkYellow,
+        Qt::darkCyan,
+        Qt::darkMagenta,
+        Qt::black,
+        Qt::darkBlue,
+        Qt::darkRed
+};
+
+class SeekSegment
+{
+public:
+    SeekSegment( seeksegment_t *seeksegment )
+    {
+        i_time_offset = seeksegment->i_time_offset;
+        i_byte_offset = seeksegment->i_byte_offset;
+        i_time_duration = seeksegment->i_time_duration;
+        i_byte_duration = seeksegment->i_byte_duration;
+        qs_layerName = QString::fromUtf8( seeksegment->psz_layerName );
+        i_eventNum = seeksegment->i_eventNum;
+        qs_text = QString::fromUtf8( seeksegment->psz_text );
+        i_layer_id = seeksegment->i_layer_id;
+    };
+    int64_t i_byte_offset;
+    int64_t i_time_offset;
+    int64_t i_byte_duration;
+    int64_t i_time_duration;
+    QString qs_layerName;
+    int     i_eventNum;
+    QString qs_text;
+    int     i_layer_id;
+};
+
+class SeekSegments : public QObject
+{
+    Q_OBJECT
+public:
+    SeekSegments( QObject *, intf_thread_t * );
+    ~SeekSegments();
+    QList<SeekSegment> const getSegments();
+    bool access() { return listMutex.tryLock( 100 ); }
+    void release() { listMutex.unlock(); }
+    char* psz_layerName;
+    bool b_med_active;
+
+public slots:
+    void update( bool isMain );
+    void updateMain();
+
+private:
+    QList<SeekSegment> segmentsList;
+    QMutex listMutex;
+    intf_thread_t *p_intf;
+};
+
+#endif // SEEKSEGMENTS_HPP
diff --git a/modules/gui/qt4/input_manager.hpp b/modules/gui/qt4/input_manager.hpp
index 49f90d3..1a8e28d 100644
--- a/modules/gui/qt4/input_manager.hpp
+++ b/modules/gui/qt4/input_manager.hpp
@@ -40,12 +40,9 @@
 #include <QEvent>
 class QSignalMapper;
 
-enum EditState
-{
-    disabled = -1,
-    segmentStart = 1,
-    segmentEnd = 2
-};
+#ifndef INT64_C
+#define INT64_C(c) (c ## LL)
+#endif
 
 enum { NORMAL,    /* loop: 0, repeat: 0 */
        REPEAT_ONE,/* loop: 0, repeat: 1 */
diff --git a/modules/gui/qt4/styles/seekstyle.cpp b/modules/gui/qt4/styles/seekstyle.cpp
index 9c90d2c..5aa40af 100644
--- a/modules/gui/qt4/styles/seekstyle.cpp
+++ b/modules/gui/qt4/styles/seekstyle.cpp
@@ -99,10 +99,16 @@ void SeekStyle::drawComplexControl( ComplexControl cc, const QStyleOptionComplex
                 backgroundGradient.setColorAt( 1.0, backgroundBase );
 
                 /* set the foreground color and gradient */
-                QColor foregroundBase( 50, 156, 255 );
-                QLinearGradient foregroundGradient( 0, 0, 0, groove.height() );
-                foregroundGradient.setColorAt( 0.0,  foregroundBase );
-                foregroundGradient.setColorAt( 1.0,  foregroundBase.darker( 125 ) );
+                QColor foregroundBase;
+                QLinearGradient foregroundGradient; 
+                if ( !slideroptions->bMedActive )  /* Normal style */
+                {
+                    foregroundBase = QColor( 50, 156, 255 );
+                    foregroundGradient = QLinearGradient( 0, 0, 0, groove.height() );
+                    foregroundGradient.setColorAt( 0.0,  foregroundBase );
+                    foregroundGradient.setColorAt( 1.0,  foregroundBase.darker( 125 ) );
+                }
+                
 
                 /* draw a slight 3d effect on the bottom */
                 painter->setPen( QColor( 230, 230, 230 ) );
@@ -120,11 +126,44 @@ void SeekStyle::drawComplexControl( ComplexControl cc, const QStyleOptionComplex
                 valueRect.setWidth( sliderPos );
 
                 /* draw foreground */
-                if ( slideroptions->sliderPosition > slideroptions->minimum && slideroptions->sliderPosition <= slideroptions->maximum )
+                if ( slideroptions->bMedActive )  /* MED mode slider */
+                {
+                    int id = slideroptions->iEventLayersId;
+                    
+                    if ( id != -1 )     /* Secondary MED slider, color always the same */
+                        foregroundBase = segmentColors[id%9];
+                    
+                    foreach( SeekSegment segment, slideroptions->segmentsList )
+                    {
+                        if ( id == -1 )
+                            foregroundBase = segmentColors[segment.i_layer_id%9];   /* Colors repeat after 9 */
+                        
+                        foregroundGradient = QLinearGradient( 0, 0, 0, groove.height() );
+                        foregroundGradient.setColorAt( 0.0,  foregroundBase );
+                        foregroundGradient.setColorAt( 1.0,  foregroundBase.darker( 125 ) );
+
+                        // draw Meta foreground
+                        painter->setPen( Qt::NoPen );
+                        painter->setBrush( foregroundGradient );
+
+                        qreal startPoint = (qreal)(segment.i_time_offset) / slideroptions->length * groove.width();
+                        qreal duration = (qreal)(segment.i_time_duration) / slideroptions->length * groove.width();
+                        
+                        QRect eventRect = groove.adjusted(startPoint , 1, -1, 0 );
+                        eventRect.setWidth( duration );
+                        
+                        painter->drawRoundedRect( eventRect, RADIUS, RADIUS );
+                    }
+                }
+                else
                 {
-                    painter->setPen( Qt::NoPen );
-                    painter->setBrush( foregroundGradient );
-                    painter->drawRoundedRect( valueRect, RADIUS, RADIUS );
+                    /* Normal foreground */
+                    if ( slideroptions->sliderPosition > slideroptions->minimum && slideroptions->sliderPosition <= slideroptions->maximum )
+                    {
+                        painter->setPen( Qt::NoPen );
+                        painter->setBrush( foregroundGradient );
+                        painter->drawRoundedRect( valueRect, RADIUS, RADIUS );
+                    }
                 }
 
                 /* draw buffering overlay */
@@ -151,7 +190,7 @@ void SeekStyle::drawComplexControl( ComplexControl cc, const QStyleOptionComplex
                 //painter->setBrush( QColor( 0, 0, 255, 150 ) );
                 //painter->drawRect( handle );
 
-                if ( option->state & QStyle::State_MouseOver || slideroptions->animate )
+                if ( ( option->state & QStyle::State_MouseOver || slideroptions->animate ) || slideroptions->bMedActive )
                 {
                     QPalette p = slideroptions->palette;
 
@@ -167,7 +206,7 @@ void SeekStyle::drawComplexControl( ComplexControl cc, const QStyleOptionComplex
                         {
                             foreach( int64_t time, slideroptions->points )
                             {
-                                int x = groove.x() + time / 1000000.0 / slideroptions->length * groove.width();
+                                int x = groove.x() + (qreal)time / (qreal)slideroptions->length * groove.width();
                                 painter->setPen( foreground );
                                 painter->setBrush( Qt::NoBrush );
                                 painter->drawLine( x, slideroptions->rect.height(), x, slideroptions->rect.height() - CHAPTERSSPOTSIZE );
@@ -175,10 +214,10 @@ void SeekStyle::drawComplexControl( ComplexControl cc, const QStyleOptionComplex
                         }
                     }
 
-                    /* draw handle */
-                    if ( option->state & QStyle::State_Enabled && sliderPos != -1 )
+                    /* draw handle (always shown for MED mode, and not for the secondary sliders) */
+                    if ( option->state & QStyle::State_Enabled && sliderPos != -1 && slideroptions->iEventLayersId == -1 )
                     {
-                        QSize hSize = QSize( handle.height(), handle.height() ) - QSize( 6, 6 );;
+                        QSize hSize = QSize( handle.height(), handle.height() ) - QSize( 6, 6 );
                         QPoint pos = QPoint( handle.center().x() - ( hSize.width() / 2 ), handle.center().y() - ( hSize.height() / 2 ) );
 
                         QPoint shadowPos( pos - QPoint( 2, 2 ) );
@@ -186,8 +225,30 @@ void SeekStyle::drawComplexControl( ComplexControl cc, const QStyleOptionComplex
 
                         /* prepare the handle's gradient */
                         QLinearGradient handleGradient( 0, 0, 0, hSize.height() );
-                        handleGradient.setColorAt( 0.0, p.window().color().lighter( 120 ) );
-                        handleGradient.setColorAt( 0.9, p.window().color().darker( 120 ) );
+                        QLinearGradient secondaryHandleGradient( 0, 0, 0, hSize.height() );
+                        QColor handleColor, secondaryHandleColor;
+
+                        if ( ( slideroptions->bMedActive ) && ( slideroptions->editState != disabled ) )
+                        {
+                            if ( slideroptions->editState == segmentEnd )
+                            {
+                                handleColor = Qt::red;
+                                secondaryHandleColor = Qt::green;
+                            }
+                            else 
+                            {
+                                handleColor = Qt::green;
+                                secondaryHandleColor = Qt::red;
+                            }
+                            
+                            secondaryHandleGradient.setColorAt( 0.0, secondaryHandleColor.lighter( 120 ) );
+                            secondaryHandleGradient.setColorAt( 0.9, secondaryHandleColor.darker( 120 ) );  
+                        }
+                        else
+                            handleColor = p.window().color();
+
+                        handleGradient.setColorAt( 0.0, handleColor.lighter( 120 ) );
+                        handleGradient.setColorAt( 0.9, handleColor.darker( 120 ) );
 
                         /* prepare the handle's shadow gradient */
                         QColor shadowBase = p.shadow().color();
@@ -204,15 +265,43 @@ void SeekStyle::drawComplexControl( ComplexControl cc, const QStyleOptionComplex
                         shadowGradient.setColorAt( 1.0, shadowLight );
 
                         painter->setPen( Qt::NoPen );
-                        painter->setOpacity( slideroptions->animationopacity );
+
+                        if ( slideroptions->bMedActive )  /* Don't hide handle in MED mode */
+                            painter->setOpacity( 1 );
+                        else
+                            painter->setOpacity( slideroptions->animationopacity );
 
                         /* draw the handle's shadow */
                         painter->setBrush( shadowGradient );
                         painter->drawEllipse( shadowPos.x(), shadowPos.y() + 1, sSize.width(), sSize.height() );
 
-                        /* finally draw the handle */
-                        painter->setBrush( handleGradient );
-                        painter->drawEllipse( pos.x(), pos.y(), hSize.width(), hSize.height() );
+                        if ( ( slideroptions->bMedActive ) && ( slideroptions->editState != disabled ) )
+                        {
+                            qreal staticSegmentLoc;
+
+                            /* draw the secondary handle's shadow */
+                            if ( slideroptions->editState == segmentEnd ) 
+                                staticSegmentLoc = groove.x() + (qreal)groove.width() * (qreal)slideroptions->editEventStartTime / slideroptions->length;
+                            else
+                                staticSegmentLoc = groove.x() + (qreal)groove.width() * (qreal)slideroptions->editEventEndTime / slideroptions->length;
+
+                            /* secondary shadow */
+                            painter->drawEllipse( staticSegmentLoc - ( hSize.width() / 2 ) - 2, shadowPos.y() + 1, sSize.width(), sSize.height() );
+
+                            /* draw the secondary handle */
+                            painter->setBrush( secondaryHandleGradient );
+                            painter->drawEllipse( staticSegmentLoc - ( hSize.width() / 2 ) , pos.y(), hSize.width(), hSize.height() );
+
+                            /* finally draw the handle */
+                            painter->setBrush( handleGradient );
+                            painter->drawEllipse( pos.x(), pos.y(), hSize.width(), hSize.height() );
+                        }
+                        else
+                        {
+                            /* finally draw the handle */
+                            painter->setBrush( handleGradient );
+                            painter->drawEllipse( pos.x(), pos.y(), hSize.width(), hSize.height() );
+                        }
                     }
                 }
             }
diff --git a/modules/gui/qt4/styles/seekstyle.hpp b/modules/gui/qt4/styles/seekstyle.hpp
index 33947bb..76372c9 100644
--- a/modules/gui/qt4/styles/seekstyle.hpp
+++ b/modules/gui/qt4/styles/seekstyle.hpp
@@ -24,9 +24,11 @@
 #define SEEKSTYLE_HPP
 
 #include <inttypes.h>
+#include "adapters/seeksegments.hpp"
 #include <QProxyStyle>
 #include <QStyleOptionSlider>
 
+
 class SeekStyle : public QProxyStyle
 {
     Q_OBJECT
@@ -36,10 +38,18 @@ public:
     public:
         SeekStyleOption();
         float buffering;
-        int length;
+        int64_t length;
         bool animate;
         qreal animationopacity;
         QList<int64_t> points;
+
+        /* MED properties */
+        bool bMedActive;
+        int iEventLayersId;
+        int editState;
+        int64_t editEventStartTime;
+        int64_t editEventEndTime;
+        QList<SeekSegment> segmentsList;
     };
 
 public:
diff --git a/modules/gui/qt4/util/input_slider.cpp b/modules/gui/qt4/util/input_slider.cpp
index d947d86..edf6956 100644
--- a/modules/gui/qt4/util/input_slider.cpp
+++ b/modules/gui/qt4/util/input_slider.cpp
@@ -30,6 +30,7 @@
 #include "qt4.hpp"
 
 #include "util/input_slider.hpp"
+#include "input_manager.hpp"
 #include "adapters/seekpoints.hpp"
 
 #include <QPaintEvent>
@@ -53,8 +54,8 @@
 #define FADEDURATION 300
 #define FADEOUTDELAY 2000
 
-SeekSlider::SeekSlider( Qt::Orientation q, QWidget *_parent, bool _static )
-          : QSlider( q, _parent ), b_classic( _static )
+SeekSlider::SeekSlider( Qt::Orientation q, QWidget *_parent, intf_thread_t * _p_intf, bool _static )
+          : QSlider( q, _parent ), b_classic( _static ), p_intf( _p_intf )
 {
     isSliding = false;
     isJumping = false;
@@ -119,6 +120,15 @@ SeekSlider::SeekSlider( Qt::Orientation q, QWidget *_parent, bool _static )
         setStyle( alternativeStyle );
     }
 
+    /* MED properties */
+    events = NULL;
+    iEventLayersId = -1;
+
+    /* Annotation Tooltip bubble */
+    mAnnotationTooltip = new AnnotationTooltip( this );
+    mAnnotationTooltip->setMouseTracking( true );
+    mAnnotationTooltip->installEventFilter( this );
+
     /* Init to 0 */
     setPosition( -1.0, 0, 0 );
     secstotimestr( psz_length, 0 );
@@ -141,6 +151,7 @@ SeekSlider::SeekSlider( Qt::Orientation q, QWidget *_parent, bool _static )
 SeekSlider::~SeekSlider()
 {
     delete chapters;
+    delete events; 
     if ( alternativeStyle )
         delete alternativeStyle;
 }
@@ -164,7 +175,7 @@ void SeekSlider::setChapters( SeekPoints *chapters_ )
  * \param time Elapsed time. Unused
  * \param legnth Duration time.
  ***/
-void SeekSlider::setPosition( float pos, int64_t time, int length )
+void SeekSlider::setPosition( float pos, int64_t time, int64_t length )
 {
     VLC_UNUSED(time);
     if( pos == -1.0 )
@@ -259,7 +270,7 @@ void SeekSlider::mousePressEvent( QMouseEvent* event )
                 int i_min_diff = i_width + 1;
                 for( int i = 0 ; i < points.count() ; i++ )
                 {
-                    int x = points.at(i).time / 1000000.0 / inputLength * i_width;
+                    int x = (qreal)points.at(i).time / inputLength * i_width;
                     int diff_x = abs( x - event->x() );
                     if ( diff_x < i_min_diff )
                     {
@@ -281,6 +292,41 @@ void SeekSlider::mousePressEvent( QMouseEvent* event )
     isSliding = true ;
 
     setValue( QStyle::sliderValueFromPosition( MINIMUM, MAXIMUM, event->x() - handleLength() / 2, width() - handleLength(), false ) );
+
+    /* Handle editing case for switching active markers */ 
+    if ( events && events->b_med_active )
+    {
+        int64_t markerLocation, curPos;
+
+        if ( editState() != disabled ) 
+        {
+            if ( editState() == segmentEnd )  //Groove length times ratio plus left x of groove
+                curPos = ( (qreal)( width() - handleLength() ) * (qreal)editEventStartTime() / inputLength ) + handleLength()/2; 
+            else
+                curPos = ( (qreal)( width() - handleLength() ) * (qreal)editEventEndTime() / inputLength ) + handleLength()/2;
+                
+            if ( abs( event->x() - curPos ) <= ( handleLength() / 2) )    //User clicked on the other marker, switch control
+            {
+                int64_t curTime;
+
+                THEMIM->getIM()->getInputTime( &curTime );
+
+                if ( editState() == segmentEnd )
+                {
+                    editState( segmentStart );
+                    emit switchEdit( false );
+                    editEventEndTime( curTime );
+                }
+                else
+                {
+                    editState( segmentEnd );
+                    emit switchEdit( true );
+                    editEventStartTime( curTime );
+                }
+            }
+        }
+    }
+
     emit sliderMoved( value() );
     event->accept();
 }
@@ -318,7 +364,7 @@ void SeekSlider::mouseMoveEvent( QMouseEvent *event )
                     b_startsnonzero = ( points.at(0).time > 0 );
                 for( int i = 0 ; i < points.count() ; i++ )
                 {
-                    int x = points.at(i).time / 1000000.0 / inputLength * size().width();
+                    int x = (qreal)points.at(i).time / inputLength * size().width();
                     if ( event->x() >= x )
                         i_selected = i + ( ( b_startsnonzero )? 1 : 0 );
                 }
@@ -329,8 +375,32 @@ void SeekSlider::mouseMoveEvent( QMouseEvent *event )
         QPoint target( event->globalX() - ( event->x() - posX ),
                   QWidget::mapToGlobal( QPoint( 0, 0 ) ).y() );
         if( likely( size().width() > handleLength() ) ) {
-            secstotimestr( psz_length, ( ( posX - margin ) * inputLength ) / ( size().width() - handleLength() ) );
+            secstotimestr( psz_length, ( ( posX - margin ) * inputLength / 1000000 ) / ( size().width() - handleLength() ) );
             mTimeTooltip->setTip( target, psz_length, chapterLabel );
+
+            if ( events && events->b_med_active )
+            {
+                /* display annotation */
+                seeksegment_t *p_segment = NULL;
+                int64_t searchTime = ( ( ( posX - margin ) * inputLength ) / ( size().width() - handleLength() ) );
+
+                if ( THEMIM->getIM()->getMedSegmentAt( &p_segment, NULL, searchTime, iEventLayersId ) != VLC_SUCCESS )
+                    return;
+                
+                if ( p_segment )
+                {
+                    /*Check layer is valid and annotation matches slider layer */
+                    if ( p_segment->psz_layerName )  
+                    {
+                        mAnnotationTooltip->setTip( target, QString(p_segment->psz_layerName), p_segment->i_eventNum , QString(p_segment->psz_text) );
+                        mAnnotationTooltip->show();
+                    }  
+
+                    vlc_seeksegment_Delete( p_segment );
+                }
+                else
+                    mAnnotationTooltip->hide();
+            }
         }
     }
     event->accept();
@@ -338,6 +408,9 @@ void SeekSlider::mouseMoveEvent( QMouseEvent *event )
 
 void SeekSlider::wheelEvent( QWheelEvent *event )
 {
+    if ( iEventLayersId != -1 ) /* ignore event on layer sliders */
+        return;
+
     /* Don't do anything if we are for somehow reason sliding */
     if( !isSliding && isEnabled() )
     {
@@ -352,6 +425,19 @@ void SeekSlider::wheelEvent( QWheelEvent *event )
 
 void SeekSlider::enterEvent( QEvent * )
 {
+    if ( events )
+    {
+        if ( events->b_med_active )
+        {
+            if( isEnabled() && inputLength > 0 && !qApp->activePopupWidget() )
+            {
+                mTimeTooltip->show();
+                mAnnotationTooltip->show();
+            }
+            return;
+        }
+    }
+
     /* Cancel the fade-out timer */
     hideHandleTimer->stop();
     /* Only start the fade-in if needed */
@@ -370,6 +456,14 @@ void SeekSlider::enterEvent( QEvent * )
 
 void SeekSlider::leaveEvent( QEvent * )
 {
+    if ( events )
+        if ( events->b_med_active )  /* MED mode */
+        {
+            mTimeTooltip->hide();
+            mAnnotationTooltip->hide();
+            return;
+        }
+
     hideHandleTimer->start();
     /* Hide the tooltip
        - if the mouse leave the slider rect (Note: it can still be
@@ -402,6 +496,16 @@ void SeekSlider::paintEvent( QPaintEvent *ev )
         option.minimum = minimum();
         if ( chapters ) foreach( const SeekPoint &point, chapters->getPoints() )
             option.points << point.time;
+
+        /* MED properties */
+        option.bMedActive =  ( ( events && events->b_med_active ) || ( editState() != disabled ) );
+        if ( events )
+            option.segmentsList = events->getSegments();
+        option.iEventLayersId = iEventLayersId;
+        option.editState = editState();
+        option.editEventStartTime = editEventStartTime();
+        option.editEventEndTime = editEventEndTime();
+
         QPainter painter( this );
         style()->drawComplexControl( QStyle::CC_Slider, &option, &painter, this );
     }
@@ -412,6 +516,7 @@ void SeekSlider::paintEvent( QPaintEvent *ev )
 void SeekSlider::hideEvent( QHideEvent * )
 {
     mTimeTooltip->hide();
+    mAnnotationTooltip->hide();
 }
 
 bool SeekSlider::eventFilter( QObject *obj, QEvent *event )
@@ -465,6 +570,10 @@ inline int SeekSlider::handleLength()
 
 void SeekSlider::hideHandle()
 {
+    if ( events )
+        if ( events->b_med_active ) /* always show handle in MED */
+            return;
+
     /* If pause is called while not running Qt will complain */
     if( animHandle->state() == QAbstractAnimation::Running )
         animHandle->pause();
@@ -473,6 +582,98 @@ void SeekSlider::hideHandle()
     animHandle->start();
 }
 
+void SeekSlider::setEvents( SeekSegments *events_ )
+{
+    delete events;
+    events = events_;
+    events->setParent( this );
+}
+
+void SeekSlider::setEditMode( EditState setEditState )
+{
+    if ( editState() == setEditState )
+        return;
+
+    if ( setEditState == disabled ) /* Close editing mode */
+    {
+        editEventStartTime( -1 );
+        editEventEndTime( -1 );
+    }
+    else
+    {
+        int64_t curTime;
+        THEMIM->getIM()->getInputTime( &curTime );
+
+        if ( editState() == disabled ) /* Start editing mode */
+        {
+            editEventStartTime( curTime );
+            editEventEndTime( curTime );
+        }
+        else  /* Switch focus for editing mode */
+        {
+            int64_t updateTime;
+            if ( setEditState == segmentStart )
+            {
+                editEventEndTime( curTime );
+                updateTime = editEventStartTime();
+            }
+            else
+            {
+                editEventStartTime( curTime );
+                updateTime = editEventEndTime();
+            }
+
+            THEMIM->getIM()->setInputTime( updateTime );
+        }
+    }
+
+    editState( setEditState );
+    repaint();
+}
+
+EditState SeekSlider::editState() const
+{
+    return THEMIM->getIM()->editState;
+}
+
+void SeekSlider::editState( EditState setState )
+{
+    THEMIM->getIM()->editState = setState;
+}
+
+int64_t SeekSlider::editEventStartTime() const
+{
+    return THEMIM->getIM()->editEventStartTime;
+}
+
+void SeekSlider::editEventStartTime( int64_t setStart )
+{
+    THEMIM->getIM()->editEventStartTime = setStart;
+}
+
+int64_t SeekSlider::editEventEndTime() const
+{
+    return THEMIM->getIM()->editEventEndTime;
+}
+
+void SeekSlider::editEventEndTime( int64_t setEnd )
+{
+    THEMIM->getIM()->editEventEndTime = setEnd;
+}
+
+void SeekSlider::setEventTimes( int64_t startTime, int64_t endTime )
+{
+    if ( !inputLength )
+        return;
+
+    if ( startTime != -1 )
+        editEventStartTime( startTime );
+
+    if ( endTime != -1 )
+        editEventEndTime( endTime );
+
+    repaint();
+}
 
 /* This work is derived from Amarok's work under GPLv2+
     - Mark Kretschmann
diff --git a/modules/gui/qt4/util/input_slider.hpp b/modules/gui/qt4/util/input_slider.hpp
index c8e48fe..b414e2c 100644
--- a/modules/gui/qt4/util/input_slider.hpp
+++ b/modules/gui/qt4/util/input_slider.hpp
@@ -32,12 +32,15 @@
 
 #include <vlc_common.h>
 #include "timetooltip.hpp"
+#include "annotation_tooltip.hpp"
 #include "styles/seekstyle.hpp"
+#include "adapters/seeksegments.hpp"
 
 #include <QSlider>
 #include <QPainter>
 #include <QTime>
 
+
 #define MSTRTIME_MAX_SIZE 22
 
 class QMouseEvent;
@@ -49,15 +52,31 @@ class QPropertyAnimation;
 class QStyleOption;
 class QCommonStyle;
 
+enum EditState
+{
+    disabled = -1,
+    segmentStart = 1,
+    segmentEnd = 2
+};
+
 /* Input Slider derived from QSlider */
 class SeekSlider : public QSlider
 {
     Q_OBJECT
     Q_PROPERTY(qreal handleOpacity READ handleOpacity WRITE setHandleOpacity)
 public:
-    SeekSlider( Qt::Orientation q, QWidget *_parent = 0, bool _classic = false );
+    SeekSlider( Qt::Orientation q, QWidget *_parent = 0, struct intf_thread_t * = NULL, bool _classic = false );
     ~SeekSlider();
     void setChapters( SeekPoints * );
+    void setEvents( SeekSegments * );
+    EditState editState() const;
+    SeekSegments* events;
+    int iEventLayersId;
+    int64_t editEventStartTime() const;
+    int64_t editEventEndTime() const;
+    void editEventStartTime( int64_t );
+    void editEventEndTime( int64_t );
+    void editState( EditState );
 
 protected:
     virtual void mouseMoveEvent( QMouseEvent *event );
@@ -77,14 +96,16 @@ protected:
     qreal handleOpacity() const;
     void setHandleOpacity( qreal opacity );
     int handleLength();
+    char psz_length[MSTRTIME_MAX_SIZE];               /* Used for the ToolTip */
+    
+    QCommonStyle *alternativeStyle;
+    TimeTooltip *mTimeTooltip;
+    int64_t inputLength;                           /* InputLength that can change */
+	bool isSliding;        /* Whether we are currently sliding by user action */
 
 private:
-    bool isSliding;        /* Whether we are currently sliding by user action */
     bool isJumping;              /* if we requested a jump to another chapter */
-    int inputLength;                           /* InputLength that can change */
-    char psz_length[MSTRTIME_MAX_SIZE];               /* Used for the ToolTip */
     QTimer *seekLimitTimer;
-    TimeTooltip *mTimeTooltip;
     float f_buffering;
     QTime bufferingStart;
     SeekPoints* chapters;
@@ -100,26 +121,33 @@ private:
     QColor tickpointForeground;
     QColor shadowDark;
     QColor shadowLight;
-    QCommonStyle *alternativeStyle;
+
 
     /* Handle's animation */
     qreal mHandleOpacity;
     QPropertyAnimation *animHandle;
     QTimer *hideHandleTimer;
 
+    /* MED */
+    AnnotationTooltip *mAnnotationTooltip;
+    struct intf_thread_t *p_intf;
+
 public slots:
-    void setPosition( float, int64_t, int );
+    void setPosition( float, int64_t, int64_t );
     void setSeekable( bool b ) { b_seekable = b ; }
     void updateBuffering( float );
     void hideHandle();
 
+    void setEditMode( EditState );
+    void setEventTimes( int64_t startTime, int64_t endTime );
+    void updatePos();
+
 private slots:
     void startSeekTimer();
-    void updatePos();
 
 signals:
     void sliderDragged( float );
-
+    void switchEdit( bool );
 };
 
 /* Sound Slider inherited directly from QAbstractSlider */
-- 
1.8.4.msysgit.0

